==> controller_generator-0.1.4.tar.gz: lib/src/validatable_builder.dart <==
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/nullability_suffix.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:analyzer/dart/element/visitor.dart';
import 'package:build/src/builder/build_step.dart';
import 'package:controller/controller.dart';
import 'package:controller_generator/src/util.dart';
import 'package:source_gen/source_gen.dart';

class ValidatableBuilder extends GeneratorForAnnotation<Validatable> {
  @override
  String generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) {
    var validatorBuilder = ValidatorBuilder();
    element.visitChildren(validatorBuilder);
    return '''
    class ${element.name}\$Validator extends EntityValidator {
      static final ${element.name}\$Validator instance = ${element.name}\$Validator();
    
      ${element.name}\$Validator() : super([${validatorBuilder.validators.join(', ')}]);
    }
    ''';
  }
}

class ValidatorBuilder extends SimpleElementVisitor<void> {
  final List<String> validators = [];

  @override
  void visitFieldElement(FieldElement element) {
    if (!element.isStatic && !element.isPrivate) {
      var validatorsForProperty = <String>{};
      if (_isRequired(element)) {
        validatorsForProperty.add('required');
      }
      validatorsForProperty.addAll(_customValidators(element));
      var type = element.type;
      validatorsForProperty.addAll(_validatorsForType(type));
      if (validatorsForProperty.isNotEmpty) {
        validators.add("PropertyValidator('${element.name}', "
            "[${validatorsForProperty.join(',')}], "
            '(entity) => entity.${element.name},'
            ')');
      }
    }
  }

  Set<String> _validatorsForType(DartType type) {
    var validators = <String>{};
    validators.addAll(_nestedValidator(type));
    validators.addAll(_iterableValidator(type));
    if (type.nullabilitySuffix == NullabilitySuffix.none) {
      validators.add('required');
    }
    return validators;
  }

  List<String> _customValidators(FieldElement element) => element.metadata
      .where((metadata) =>
          isType(metadata.computeConstantValue()!.type!, Validator))
      .map((metadata) => metadata.toSource().substring(1))
      .toList();

  List<String> _nestedValidator(DartType type) =>
      hasMeta(type.element!, Validatable)
          ? ['${type.element!.name}\$Validator.instance']
          : [];

  List<String> _iterableValidator(DartType type) =>
      type.isDartCoreSet || type.isDartCoreList
          ? [
              'IterableValidator(CompositeValidator(['
                  '${_validatorsForType((type as ParameterizedType).typeArguments[0]).join(',')}]))'
            ]
          : [];

  bool _isRequired(FieldElement element) =>
      element.type.nullabilitySuffix == NullabilitySuffix.none;
}

==> controller_generator-0.1.9.tar.gz: lib/src/validatable_builder.dart <==
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/nullability_suffix.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:analyzer/dart/element/visitor.dart';
import 'package:build/build.dart';
import 'package:controller/controller.dart';
import 'package:controller_generator/src/util.dart';
import 'package:source_gen/source_gen.dart';

class ValidatableBuilder extends GeneratorForAnnotation<Validatable> {
  @override
  String generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) {
    var validatorBuilder = ValidatorBuilder();
    element.visitChildren(validatorBuilder);
    return '''
    class ${element.name}\$Validator extends EntityValidator {
      static final ${element.name}\$Validator instance = ${element.name}\$Validator();
    
      ${element.name}\$Validator() : super([${validatorBuilder.validators.join(', ')}]);
    }
    ''';
  }
}

class ValidatorBuilder extends SimpleElementVisitor<void> {
  final List<String> validators = [];

  @override
  void visitFieldElement(FieldElement element) {
    if (!element.isStatic && !element.isPrivate && !element.isSynthetic) {
      var validatorsForProperty = <String>{};
      if (_isRequired(element)) {
        validatorsForProperty.add('required');
      }
      validatorsForProperty.addAll(_customValidators(element));
      var type = element.type;
      validatorsForProperty.addAll(_validatorsForType(type));
      if (validatorsForProperty.isNotEmpty) {
        validators.add("PropertyValidator('${element.name}', "
            "[${validatorsForProperty.join(',')}], "
            '(entity) => entity.${element.name},'
            ')');
      }
    }
  }

  Set<String> _validatorsForType(DartType type) {
    var validators = <String>{};
    validators.addAll(_nestedValidator(type));
    validators.addAll(_iterableValidator(type));
    if (type.nullabilitySuffix == NullabilitySuffix.none) {
      validators.add('required');
    }
    return validators;
  }

  List<String> _customValidators(FieldElement element) => element.metadata
      .where((metadata) =>
          isType(metadata.computeConstantValue()!.type!, Validator))
      .map((metadata) => metadata.toSource().substring(1))
      .toList();

  List<String> _nestedValidator(DartType type) =>
      hasMeta(type.element!, Validatable)
          ? ['${type.element!.name}\$Validator.instance']
          : [];

  List<String> _iterableValidator(DartType type) =>
      type.isDartCoreSet || type.isDartCoreList
          ? [
              'IterableValidator(CompositeValidator(['
                  '${_validatorsForType((type as ParameterizedType).typeArguments[0]).join(',')}]))'
            ]
          : [];

  bool _isRequired(FieldElement element) =>
      element.type.nullabilitySuffix == NullabilitySuffix.none;
}
